import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AccountService, CommercialPremiseService, FlatService, ParkingService, PremiseService, TenancyService, TenantService } from '../data-object-service';
import { Account, CommercialPremise, Flat, NULL_Account, NULL_CommercialPremise, NULL_Flat, NULL_Parking, NULL_Tenant, Parking, Premise, Tenancy, Tenant } from '../data-objects';
import { MessageService } from '../message.service';
import { MatButton } from '@angular/material/button';
import { MatTableDataSource } from '@angular/material/table';


interface DN_Tenancy {
  rawTenancy: Tenancy
  flat: string
  parking: string
  commercialPremise: string  
}

@Component({
  selector: 'app-tenant-details',
  templateUrl: './tenant-details.component.html',
  styleUrls: ['./tenant-details.component.css']
})
export class TenantDetailsComponent implements OnInit {

  tenant: Tenant = NULL_Tenant

  account: Account = NULL_Account

  tenancies: DN_Tenancy[] = []
  tenancyDataSource: MatTableDataSource<DN_Tenancy>
  tenancyDisplayColumns: string[] = [ "description", "flat", "parking", "commercial_premise", "startdate", "enddate" ]

  collapseTenantDetails: boolean = false
  collapseTenancies: boolean = false

  @ViewChild('submitButton') submitButton: MatButton

  constructor(
    private tenantService: TenantService, 
    private accountService: AccountService,
    private tenancyService: TenancyService,
    private flatService: FlatService,
    private parkingService: ParkingService,
    private commercialPremiseService: CommercialPremiseService,
    private premiseService: PremiseService,
    private messageService: MessageService,
    private route: ActivatedRoute,
    private router: Router
  ) { }

  async getTenant(): Promise<void> {
    try {
      const id = +this.route.snapshot.paramMap.get('id')
      if (id != 0) {
        this.tenant = await this.tenantService.getTenant(id)
        this.account = await this.accountService.getAccount(this.tenant.account)

        const premises: Premise[] = await this.premiseService.getPremises()
        const premisesDict = new Map<number, Premise>()
        for (let p of premises) {
          premisesDict.set(p.id, p)
        }
        for (let t of await this.tenancyService.getTenancysByTenant(this.tenant.id)) {
          const flat: Flat = (t.flat) ? await this.flatService.getFlat(t.flat) : NULL_Flat
          const parking: Parking = (t.parking) ? await this.parkingService.getParking(t.parking) : NULL_Parking
          const commercialPremise: CommercialPremise = (t.commercial_premise) ? await this.commercialPremiseService.getCommercialPremise(t.commercial_premise) : NULL_CommercialPremise
          this.tenancies.push({
            rawTenancy: t,
            flat: (flat != NULL_Flat) ? `${flat.description} (${premisesDict.get(flat.premise).description})` : '',
            parking: (parking != NULL_Parking) ? `${parking.description} (${premisesDict.get(parking.premise).description})` : '',
            commercialPremise: (commercialPremise != NULL_CommercialPremise) ? `${commercialPremise.description} (${premisesDict.get(commercialPremise.premise).description})` : ''
          })
        }
        this.tenancyDataSource = new MatTableDataSource<DN_Tenancy>(this.tenancies)
      }
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async saveTenant() {
    try {
      this.submitButton.disabled = true
      this.messageService.add("saveTenant")
      this.messageService.add(JSON.stringify(this.tenant, undefined, 4))
      if (this.tenant.id == 0) {
        this.messageService.add("about to insert new tenant")
        this.account = { 
          "id": 0, 
          "description": `account_${this.tenant.firstname}_${this.tenant.lastname}`
        }
        this.account = await this.accountService.postAccount(this.account)
        this.tenant.account = this.account.id
        this.tenant = await this.tenantService.postTenant(this.tenant)
        this.messageService.add(`Successfully added account with id ${this.account.id} and tenant with id ${this.tenant.id}`)
      } else {
        this.messageService.add("about to update existing tenant")
        this.tenant = await this.tenantService.putTenant(this.tenant)
        this.messageService.add(`Successfully changed tenant with id ${this.tenant.id}`)
      }
      this.router.navigate(['/tenants'])
    } finally {
      this.submitButton.disabled = false
    }
  }

  ngOnInit(): void {
    this.getTenant()
  }

}
